# -*- coding: utf-8 -*-

# Copyright 2019 Mike Fährmann
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.

"""Extractors for https://www.mangoxo.com/"""

from .common import Extractor, Message
from .. import text, exception
from ..cache import cache


class MangoxoExtractor(Extractor):
    """Base class for mangoxo extractors"""
    category = "mangoxo"
    root = "https://www.mangoxo.com"
    cookiedomain = "www.mangoxo.com"
    cookienames = ("SESSION",)
    _warning = True

    def login(self):
        username, password = self._get_auth_info()
        if username:
            self._update_cookies(self._login_impl(username, password))
        elif MangoxoExtractor._warning:
            MangoxoExtractor._warning = False
            self.log.warning("Unauthenticated users cannot see "
                             "more than 5 images per album")

    @cache(maxage=3*3600, keyarg=1)
    def _login_impl(self, username, password):
        self.log.info("Logging in as %s", username)

        page = self.request(self.root + "/login/").text
        token = text.extract(page, 'id="loginToken" value="', '"')[0]
        if not token:
            self.log.debug("failed to extract 'loginToken'")

        url = self.root + "/login/loginxmm"
        headers = {
            "X-Requested-With": "XMLHttpRequest",
            "Referer": self.root + "/login",
        }
        data = {
            "name": username,
            "password": password,
            "loginToken": token,
        }
        response = self.request(url, method="POST", headers=headers, data=data)

        if response.json().get("result") != "1":
            raise exception.AuthenticationError()
        return {"SESSION": self.session.cookies.get("SESSION")}

    @staticmethod
    def _total_pages(page):
        return text.parse_int(text.extract(page, "total :", ",")[0])


class MangoxoAlbumExtractor(MangoxoExtractor):
    """Extractor for albums on mangoxo.com"""
    subcategory = "album"
    filename_fmt = "{album[id]}_{num:>03}.{extension}"
    directory_fmt = ("{category}", "{channel[name]}", "{album[name]}")
    archive_fmt = "{album[id]}_{num}"
    pattern = r"(?:https?://)?(?:www\.)?mangoxo\.com/album/(\w+)"
    test = ("https://www.mangoxo.com/album/lzVOv1Q9", {
        "url": "ad921fe62663b06e7d73997f7d00646cab7bdd0d",
        "keyword": {
            "channel": {
                "id": "QeYKRkO0",
                "name": "美女图社",
                "cover": str,
            },
            "album": {
                "id": "lzVOv1Q9",
                "name": "池永康晟 Ikenaga Yasunari 透出古朴气息的日本美女人像画作",
                "date": "2019.3.22 14:42",
                "description": str,
            },
            "num": int,
            "count": 65,
        },
    })

    def __init__(self, match):
        MangoxoExtractor.__init__(self, match)
        self.album_id = match.group(1)

    def items(self):
        self.login()
        url = "{}/album/{}/".format(self.root, self.album_id)
        page = self.request(url).text
        data = self.metadata(page)
        imgs = self.images(url, page)

        yield Message.Version, 1
        yield Message.Directory, data
        for data["num"], image in enumerate(imgs, 1):
            yield Message.Url, image, text.nameext_from_url(image, data)

    def metadata(self, page):
        """Return general metadata"""
        title, pos = text.extract(page, '<title>', '</title>')
        count, pos = text.extract(page, 'id="pic-count">', '<', pos)
        cover, pos = text.extract(page, ' src="', '"', pos)
        cid  , pos = text.extract(page, '//www.mangoxo.com/channel/', '"', pos)
        cname, pos = text.extract(page, '>', '<', pos)
        date , pos = text.extract(page, '</i>', '<', pos)
        descr, pos = text.extract(page, '<pre>', '</pre>', pos)

        return {
            "channel": {
                "id": cid,
                "name": text.unescape(cname),
                "cover": cover,
            },
            "album": {
                "id": self.album_id,
                "name": text.unescape(title),
                "date": date.strip(),
                "description": text.unescape(descr),
            },
            "count": text.parse_int(count),
        }

    def images(self, url, page):
        """Generator; Yields all image URLs"""
        total = self._total_pages(page)
        num = 1

        while True:
            yield from text.extract_iter(
                page, 'class="lightgallery-item" href="', '"')
            if num >= total:
                return
            num += 1
            page = self.request(url + str(num)).text


class MangoxoChannelExtractor(MangoxoExtractor):
    """Extractor for all albums on a mangoxo channel"""
    subcategory = "channel"
    pattern = r"(?:https?://)?(?:www\.)?mangoxo\.com/channel/(\w+)"
    test = ("https://www.mangoxo.com/channel/QeYKRkO0", {
        "pattern": MangoxoAlbumExtractor.pattern,
        "range": "1-30",
        "count": "> 20",
    })

    def __init__(self, match):
        MangoxoExtractor.__init__(self, match)
        self.channel_id = match.group(1)

    def items(self):
        self.login()
        num = total = 1
        url = "{}/channel/{}/album/".format(self.root, self.channel_id)
        yield Message.Version, 1

        while True:
            page = self.request(url + str(num)).text

            for album in text.extract_iter(
                    page, '<a class="link black" href="', '"'):
                yield Message.Queue, album, {}

            if num == 1:
                total = self._total_pages(page)
            if num >= total:
                return
            num += 1



# -*- coding: utf-8 -*-

# Copyright 2019 Mike Fährmann
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License version 2 as
# published by the Free Software Foundation.

"""Extractors for https://pururin.io/"""

from .common import GalleryExtractor
from .. import text, util
import json


class PururinGalleryExtractor(GalleryExtractor):
    """Extractor for image galleries on pururin.io"""
    category = "pururin"
    pattern = r"(?:https?://)?(?:www\.)?pururin\.io/(?:gallery|read)/(\d+)"
    test = ("https://pururin.io/gallery/38661/iowant-2", {
        "pattern": r"https://cdn.pururin.io/assets/images/data/38661/\d+\.jpg",
        "keyword": {
            "title"     : "Iowant 2!!",
            "title_en"  : "Iowant 2!!",
            "title_jp"  : "",
            "gallery_id": 38661,
            "count"     : 19,
            "artist"    : ["Shoda Norihiro"],
            "group"     : ["Obsidian Order"],
            "parody"    : ["Kantai Collection"],
            "characters": ["Iowa", "Teitoku"],
            "tags"      : list,
            "type"      : "Doujinshi",
            "collection": "",
            "convention": "C92",
            "rating"    : float,
            "uploader"  : "demo",
            "scanlator" : "",
            "lang"      : "en",
            "language"  : "English",
        }
    })
    root = "https://pururin.io"

    def __init__(self, match):
        self.gallery_id = match.group(1)
        url = "{}/gallery/{}/x".format(self.root, self.gallery_id)
        GalleryExtractor.__init__(self, match, url)

        self._ext = ""
        self._cnt = 0

    def metadata(self, page):
        data = text.extract_all(page, (
            ("artist"    , "<td>Artist</td>"    , "</td>"),
            ("group"     , "<td>Circle</td>"    , "</td>"),
            ("parody"    , "<td>Parody</td>"    , "</td>"),
            ("tags"      , "<td>Contents</td>"  , "</td>"),
            ("type"      , "<td>Category</td>"  , "</td>"),
            ("characters", "<td>Character</td>" , "</td>"),
            ("collection", "<td>Collection</td>", "</td>"),
            ("language"  , "<td>Language</td>"  , "</td>"),
            ("scanlator" , "<td>Scanlator</td>" , "</td>"),
            ("convention", "<td>Convention</td>", "</td>"),
            ("uploader"  , "<td>Uploader</td>"  , "</td>"),
            ("rating"    , " :rating='"         , "'"),
        ))[0]

        url = "{}/read/{}/01/x".format(self.root, self.gallery_id)
        page = self.request(url).text
        info = json.loads(text.unescape(text.extract(
            page, ':gallery="', '"')[0]))

        self._ext = info["image_extension"]
        self._cnt = info["total_pages"]

        for key in ("artist", "group", "parody", "tags", "characters"):
            data[key] = [
                text.unescape(item)
                for item in text.extract_iter(data[key], 'title="', '"')
            ]
        for key in ("type", "collection", "language", "scanlator",
                    "convention"):
            data[key] = text.unescape(text.extract(
                data[key], 'title="', '"')[0] or "")

        data["gallery_id"] = text.parse_int(self.gallery_id)
        data["title"] = info["title"] or info.get("j_title") or ""
        data["title_en"] = info["title"]
        data["title_jp"] = info.get("j_title") or ""
        data["uploader"] = text.remove_html(data["uploader"])
        data["rating"] = text.parse_float(data["rating"])
        data["lang"] = util.language_to_code(data["language"])
        return data

    def images(self, _):
        ufmt = "https://cdn.pururin.io/assets/images/data/{}/{{}}.{}".format(
            self.gallery_id, self._ext)
        return [(ufmt.format(num), None) for num in range(1, self._cnt + 1)]
